SystemOrganization addCategory: #'Croquet-Contacts'!

Object subclass: #TBroadcasterStandalone
	instanceVariableNames: 'broadcaster webserver mutex sockets'
	classVariableNames: 'Default DefaultPort'
	poolDictionaries: ''
	category: 'Croquet-Contacts'!

!TBroadcasterStandalone commentStamp: 'sn 2/26/2018 15:22' prior: 0!
Broadcaster based on WebSockets!

!TBroadcasterStandalone class methodsFor: 'as yet unclassified' stamp: 'sn 2/24/2018 18:51'!
default
	"A default WebServer instance. Mostly for the examples."

	^Default ifNil:[Default := self new].! !

!TBroadcasterStandalone class methodsFor: 'class initialization' stamp: 'sn 2/24/2018 18:51'!
initialize
	"TContactPoint initialize"
	DefaultPort := 8080.! !

!TBroadcasterStandalone class methodsFor: 'as yet unclassified' stamp: 'sn 2/24/2018 18:51'!
reset		"WebServer reset"
	"Reset the default WebServer instance."

	Default ifNotNil:[
		Default destroy.
		Default := nil.
	].! !

!TBroadcasterStandalone methodsFor: 'initialize' stamp: 'sn 2/25/2018 11:26'!
destroy

	webserver ifNotNil:[
		webserver destroy.
		webserver := nil
	].
! !

!TBroadcasterStandalone methodsFor: 'initialize' stamp: 'sn 2/25/2018 17:53'!
initialize
	super initialize.
	mutex := TMutex new.
	sockets := OrderedCollection new.

	self startBroadcaster.! !

!TBroadcasterStandalone methodsFor: 'websocket' stamp: 'sn 10/25/2018 01:40'!
runBroadcaster

	webserver := WebServer new.
	webserver listenOn: 8080 "interface: (NetNameResolver addressFromString: '10.0.1.6')".

	webserver addService: '/' action:[:req|
		req send200Response: '<div>Croquet TContactPoint broadcaster</div>' contentType: 'text/html'.
	].
	
	webserver addService: '/broadcast' action:[:req |
		|ws|	
		"Implements the WebSocket broadcaster"
		ws := req asWebSocket.
		ws timeout: 45.
		Transcript cr; show: 'New ', ws.

		ws onMessage:[:data|
			
			mutex critical:[sockets do:[:s| s send: data]].
		].
		ws onClose:[
			Transcript cr; show: 'Closing ', ws.
			mutex critical:[sockets remove: ws ifAbsent:[]].
		].
		ws onError:[:ex|

			Transcript cr; show: ws.
			Transcript cr; show: ex description.
			Transcript cr; show: ex signalerContext longStack.
		].
	
		mutex critical:[sockets add: ws].
		ws run
		
]! !

!TBroadcasterStandalone methodsFor: 'as yet unclassified' stamp: 'sn 2/25/2018 18:41'!
startBroadcaster
	self stopBroadcaster.
	broadcaster := self fork: #runBroadcaster at: Processor lowIOPriority.
! !

!TBroadcasterStandalone methodsFor: 'as yet unclassified' stamp: 'sn 2/25/2018 17:44'!
stopBroadcaster

	broadcaster ifNotNil:[
		self destroy.
		broadcaster terminate].
	broadcaster := nil.! !

Object subclass: #TContact
	instanceVariableNames: 'address port name id time buffer viewpoint viewpointName controller'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Croquet-Contacts'!

!TContact commentStamp: '<historical>' prior: 0!
The contact for a Croquet island.!

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 15:40'!
address

	^address.! !

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 15:46'!
address: hostAddress

	address := hostAddress.
	self flushBuffer.! !

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 16:17'!
address: hostAddress buffer: aByteArray count: count

	address := hostAddress.
	buffer := aByteArray copy.
	port := buffer unsignedShortAt: 5 bigEndian: true.
	id := TObjectID readHexFrom: (buffer copyFrom: 7 to: 38) asString.
	name := (buffer copyFrom: 39 to: count) asString.
	controller := nil.! !

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 15:46'!
address: hostAddress port: hostPort id: uid name: aString

	address := hostAddress.
	port := hostPort.
	id := uid.
	name:= aString.
	time := Time now.
	self flushBuffer.! !

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 15:48'!
buffer
	^buffer ifNil:[buffer := self createNewBuffer]! !

!TContact methodsFor: 'accessing' stamp: 'das 11/20/2005 20:13'!
controller

	^ controller.! !

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 16:14'!
createNewBuffer
	| buf |
	buf :=('CRQT','  ',id hex, name)asByteArray.
	buf unsignedShortAt: 5 put: port bigEndian: true.
	^buf! !

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 15:47'!
flushBuffer
	buffer := nil.! !

!TContact methodsFor: 'accessing' stamp: 'sn 2/25/2018 01:52'!
fromData: data

	address := ByteArray readHexFrom: (data at: 'address').
	port := data at: 'port'.
	id := TObjectID readHexFrom: (data at: 'id').
	name := data at: 'name'.
	controller := nil.
	self flushBuffer.! !

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 15:43'!
id

	^ id.! !

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 15:46'!
id: uid

	id := uid.
	self flushBuffer.! !

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 15:44'!
name

	^ name.! !

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 15:47'!
name: aString

	name := aString.
	self flushBuffer.! !

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 15:42'!
port

	^ port.! !

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 15:47'!
port: hostPort

	port := hostPort.
	self flushBuffer.! !

!TContact methodsFor: 'accessing' stamp: 'das 1/6/2006 15:44'!
postcard

	^TPostcard new address: address port: port id: id name: name viewpointName: viewpointName.! !

!TContact methodsFor: 'accessing' stamp: 'das 11/10/2005 13:33'!
printOn: aStream
	self address do:[:x| aStream print: x] separatedBy:[aStream nextPut: $.].
	aStream nextPut: $:.
	aStream print: self port.
	aStream print:' '.
	aStream print: self id.
	aStream nextPut: $:.
	aStream print: self name.

! !

!TContact methodsFor: 'action' stamp: 'ar 1/30/2006 23:54'!
setupContact
	| p  |
	controller ifNotNil:[ ^ self ].
	controller := TSimpleController new.
	controller connectTo: address port: port sessionID: id.

	p := controller login: 'foo' password: 'bar'.
	p wait.
	p := controller join. 	"request messages"
	p wait.
	p := controller sync. 	"request replicated state"
	p wait.
	controller install: p result.	"install replicated state"

	controller heartbeat: 20. "set heartbeat rate"
! !

!TContact methodsFor: 'action' stamp: 'ar 1/30/2006 23:54'!
setupContact: vpn
	| p  |
	controller ifNotNil:[ ^ self ].
	viewpointName := vpn.
	controller := TSimpleController new.
	controller connectTo: address port: port sessionID: id.

	p := controller login: 'foo' password: 'bar'.
	p wait.
	p := controller join. 	"request messages"
	p wait.
	p := controller sync. 	"request replicated state"
	p wait.
	controller install: p result.	"install replicated state"
	controller heartbeat: 20. "set heartbeat rate"
	p := controller island future at: viewpointName.
	p whenResolved:[:vp| viewpoint := vp].
! !

!TContact methodsFor: 'accessing' stamp: 'das 11/20/2005 17:10'!
viewpoint

	^ viewpoint.! !

!TContact methodsFor: 'accessing' stamp: 'das 11/20/2005 17:10'!
viewpoint: vp

	viewpoint := vp.! !

Object subclass: #TContactPoint
	instanceVariableNames: 'broadcastContacts udpSocket udpListener mutex broadcaster listener contactsByName contactsByID'
	classVariableNames: 'DefaultPort'
	poolDictionaries: ''
	category: 'Croquet-Contacts'!

!TContactPoint commentStamp: '<historical>' prior: 0!
I am a trivial little contact point for Croquet. I broadcast a single port that another Croquet system can use to connect itself to. I am NOT an industrial strength rendevouz server.!

!TContactPoint class methodsFor: 'class initialization' stamp: 'das 11/3/2005 22:21'!
initialize
	"TContactPoint initialize"
	DefaultPort := 43215.! !

!TContactPoint methodsFor: 'accessing' stamp: 'das 12/2/2005 11:02'!
addBroadcast: bc

	broadcastContacts add: bc.
! !

!TContactPoint methodsFor: 'accessing' stamp: 'ar 11/13/2006 14:48'!
addContact: cc 
	contactsByName at: cc name ifAbsentPut:[ cc ].
	contactsByID at: cc id ifAbsentPut:[ cc ].! !

!TContactPoint methodsFor: 'accessing' stamp: 'das 11/10/2005 12:23'!
contactsByID
	^contactsByID! !

!TContactPoint methodsFor: 'accessing' stamp: 'das 11/10/2005 12:25'!
contactsByName
	^contactsByName! !

!TContactPoint methodsFor: 'initialize' stamp: 'das 11/3/2005 16:28'!
destroy
	self stopUdpListener.
	self stopBroadcaster.
	udpSocket ifNotNil:[udpSocket destroy].
	udpSocket := nil.! !

!TContactPoint methodsFor: 'initialize' stamp: 'hrs 4/19/2006 08:51'!
initialize
	super initialize.
	mutex := TMutex new.
	contactsByName := Dictionary new.
	contactsByID := Dictionary new.
	broadcastContacts := OrderedCollection new.
	udpSocket := Socket newUDP.
	udpSocket setOption: 'SO_REUSEADDR' value: 1.
	udpSocket setOption: 'SO_REUSEPORT' value: 1.
	udpSocket setPort: DefaultPort.   "magic listening port"
	udpSocket setOption: 'SO_BROADCAST' value: 1.
	self startUdpListener.
	self startBroadcaster.! !

!TContactPoint methodsFor: 'broadcaster' stamp: 'das 11/14/2005 19:47'!
runBroadcaster
	| delay address |
	address := ByteArray with: 255 with: 255 with: 255 with: 255. "broadcast"
	delay := Delay forSeconds: 5.

	[true] whileTrue:[
		broadcastContacts do:[:bc |
			self sendPacket: bc buffer size: bc buffer size address: address port: DefaultPort.
		].
		delay wait.
	].! !

!TContactPoint methodsFor: 'udp listener' stamp: 'sn 2/24/2018 17:46'!
runUdpListener
	| buffer count address port cc |
	buffer := ByteArray new: 2048.
	[true] whileTrue:[
	
		udpSocket waitForData.
		"Note: The following is an oddity in the Squeak socket implementation. 
		The remote (peer) address/port will be set if a datagram has been received."
		mutex critical:[
			self halt.
			count := [udpSocket receiveSomeDataInto: buffer startingAt: 1]
				on: Error do:[:ex| ex return: -1].
			address := udpSocket remoteAddress.
			port := udpSocket remotePort.
		].
		count > 40 ifTrue:[
			(buffer copyFrom:1 to:4) asString = 'CRQT' ifTrue:[
				"Grab the port"
				port := buffer unsignedShortAt: 5 bigEndian: true.
				" is this one of mine?"
				(broadcastContacts findFirst:[:bc | bc port = port]) = 0 ifTrue:[ 
					cc := TContact new address: address buffer: buffer count: count.
					contactsByName at: cc name ifAbsentPut:[ cc ].
					contactsByID at: cc id ifAbsentPut:[ cc ].
				].
			].
		].
	].! !

!TContactPoint methodsFor: 'broadcaster' stamp: 'sn 2/24/2018 17:43'!
sendPacket: data size: size address: address port: port
	| count |

	mutex critical:[
		udpSocket setPeer: address port: port.
		count := [udpSocket sendSomeData: data startIndex: 1 count: size] 
			on: Error do:[:ex| 
				ex return: -1].
	].
	^count! !

!TContactPoint methodsFor: 'broadcaster' stamp: 'sn 2/25/2018 16:01'!
startBroadcaster
	self stopBroadcaster.
	broadcaster := self fork: #runBroadcaster at: Processor userInterruptPriority.! !

!TContactPoint methodsFor: 'udp listener' stamp: 'sn 2/25/2018 16:01'!
startUdpListener
	self stopUdpListener.
	udpListener := self fork: #runUdpListener at: Processor userInterruptPriority.! !

!TContactPoint methodsFor: 'broadcaster' stamp: 'das 11/3/2005 15:31'!
stopBroadcaster
	broadcaster ifNotNil:[broadcaster terminate].
	broadcaster := nil.! !

!TContactPoint methodsFor: 'udp listener' stamp: 'das 11/3/2005 15:26'!
stopUdpListener
	udpListener ifNotNil:[udpListener terminate].
	udpListener := nil.! !

Object subclass: #TContactPointWS
	instanceVariableNames: 'broadcastContacts udpSocket udpListener broadcaster listener contactsByName contactsByID wsListener wsBroadcaster mutex hostName'
	classVariableNames: 'DefaultPort'
	poolDictionaries: ''
	category: 'Croquet-Contacts'!

!TContactPointWS commentStamp: 'sn 2/26/2018 15:23' prior: 0!
I am a trivial little contact point for Croquet. I broadcast a single port that another Croquet system can use to connect itself to. I am NOT an industrial strength rendevouz server.

Based on WebSockets!

!TContactPointWS class methodsFor: 'class initialization' stamp: 'das 11/3/2005 22:21'!
initialize
	"TContactPoint initialize"
	DefaultPort := 43215.! !

!TContactPointWS methodsFor: 'accessing' stamp: 'das 12/2/2005 11:02'!
addBroadcast: bc

	broadcastContacts add: bc.
! !

!TContactPointWS methodsFor: 'accessing' stamp: 'ar 11/13/2006 14:48'!
addContact: cc 
	contactsByName at: cc name ifAbsentPut:[ cc ].
	contactsByID at: cc id ifAbsentPut:[ cc ].! !

!TContactPointWS methodsFor: 'accessing' stamp: 'das 11/10/2005 12:23'!
contactsByID
	^contactsByID! !

!TContactPointWS methodsFor: 'accessing' stamp: 'das 11/10/2005 12:25'!
contactsByName
	^contactsByName! !

!TContactPointWS methodsFor: 'initialize' stamp: 'sn 2/24/2018 20:38'!
destroy
	self stopUdpListener.
	self stopBroadcaster.
! !

!TContactPointWS methodsFor: 'accessing' stamp: 'sn 10/24/2018 23:36'!
hostName
	^hostName! !

!TContactPointWS methodsFor: 'accessing' stamp: 'sn 10/24/2018 23:36'!
hostName: aName
	hostName := aName! !

!TContactPointWS methodsFor: 'initialize' stamp: 'sn 10/24/2018 23:39'!
initialize

	super initialize.
	mutex := TMutex new.
	contactsByName := Dictionary new.
	contactsByID := Dictionary new.
	broadcastContacts := OrderedCollection new.
	
	self hostName: 'http://localhost:8080/broadcast'
	! !

!TContactPointWS methodsFor: 'broadcaster' stamp: 'sn 10/25/2018 01:39'!
runBroadcaster
	| delay |
	
	"wsBroadcaster := WebClient webSocketTo: 'http://localhost:8080/broadcast'."
	delay := Delay forSeconds: 5.

	[true] whileTrue:[
		broadcastContacts do:[:bc |
			|obj sendObj ws |
			
			ws := WebClient webSocketTo: self hostName "'http://localhost:8080/broadcast'".
			sendObj := Dictionary new.
			sendObj  at: 'msg' put: 'CRQT';
							at: 'id' put: bc id hex;
							at: 'name' put: bc name;
							at: 'address' put:  bc address hex;
							at: 'port' put: bc port;
							yourself.
							
				obj :=	WebUtils jsonEncode: sendObj.
				"Transcript show: 'send contact'; cr. "
				ws send: obj.
				ws close.
	].

		delay wait.
	].! !

!TContactPointWS methodsFor: 'udp listener' stamp: 'sn 10/24/2018 23:37'!
runUdpListener

	wsListener := WebClient webSocketTo: self hostName "'http://localhost:8080/broadcast'".
	wsListener onMessage:[:data| 
		
		|decode cc|

		decode := WebUtils jsonDecode: data readStream.
		
					"Transcript show: 'get contact - ', decode; cr."
		
					cc := TContact new fromData: decode.
					contactsByName at: cc name ifAbsentPut:[ cc ].
					contactsByID at: cc id ifAbsentPut:[ cc ].
		].
	
		wsListener run

! !

!TContactPointWS methodsFor: 'initialize' stamp: 'sn 10/24/2018 23:38'!
start

	self startBroadcaster.
	self startUdpListener.
	! !

!TContactPointWS methodsFor: 'broadcaster' stamp: 'sn 2/25/2018 17:41'!
startBroadcaster
	self stopBroadcaster.
	broadcaster := self fork: #runBroadcaster at: Processor userInterruptPriority."lowIOPriority".! !

!TContactPointWS methodsFor: 'udp listener' stamp: 'sn 2/26/2018 15:24'!
startUdpListener
	self stopUdpListener.
	udpListener := self fork: #runUdpListener at: Processor userInterruptPriority. "lowIOPriority."
! !

!TContactPointWS methodsFor: 'broadcaster' stamp: 'sn 2/25/2018 13:15'!
stopBroadcaster
	broadcaster ifNotNil:[
		wsBroadcaster ifNotNil:[wsBroadcaster close].
		broadcaster terminate].
	broadcaster := nil.! !

!TContactPointWS methodsFor: 'udp listener' stamp: 'sn 2/25/2018 13:14'!
stopUdpListener
	udpListener ifNotNil:[
		wsListener ifNotNil:[wsListener close.].
		udpListener terminate].
	udpListener := nil.! !

Object subclass: #TPostcard
	instanceVariableNames: 'routerAddress routerID routerName viewpointName viewpointID standin'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Croquet-Contacts'!

!TPostcard commentStamp: '<historical>' prior: 0!
TPostCard is used to find a Croquet router. The method it uses is:

Looks up the routerAddress. If this is a valid address, it will initiate a connection. If it is nil, it will look for the router matching the routerName/routerID on the LAN.

Once the router has been discovered, the Island is synced.

These instance variables MUST be left as references. Every element in them must exist inside of an Island that may be replicated. There can be no controllers or FarRefs placed into this object. The standin must be constructed by the containing Island.

routerAddress
routerID - the routerID is a unique ID associated with a single Island with which it should share the same ID and name.
routerName - the routerName is a user created name that can describe the Island/Router group and help the user find the router again.
viewpointName - the name of an entry point into the island. This will typically be a TFrame of some sort.
viewpointIndex - this is a name constructed of (routerID,viewpointName) and is used to index into a dictionary of viewpoints when rendering. It is a unique name that can be replicated.
standin!

!TPostcard class methodsFor: 'as yet unclassified' stamp: 'das 11/14/2005 10:53'!
howToPassAsArgument
	^#passByClone:! !

!TPostcard methodsFor: 'testing' stamp: 'das 12/7/2005 17:56'!
isFrame

	^ false.! !

!TPostcard methodsFor: 'testing' stamp: 'das 11/14/2005 09:59'!
isPortal

	^ false.! !

!TPostcard methodsFor: 'testing' stamp: 'das 12/7/2005 17:55'!
isPostcard

	^ true.! !

!TPostcard methodsFor: 'testing' stamp: 'bgf 3/20/2006 17:47'!
isTexture
	^ false! !

!TPostcard methodsFor: 'copying' stamp: 'jcg 3/30/2006 20:33'!
postCopy
	routerAddress := routerAddress copy.
	routerID := routerID copy.
	routerName := routerName copy.
	viewpointName := viewpointName copy.
	viewpointID := viewpointID copy.
	standin := standin copy.! !

!TPostcard methodsFor: 'printing' stamp: 'das 11/30/2005 10:52'!
printOn: aStream

	super printOn: aStream.
	routerAddress ifNotNil:[ aStream print: routerAddress; nextPutAll: ' : '.].
	aStream
		print: routerID;
		nextPutAll: ':';
		print: routerName;
		nextPutAll: '-';
		print: viewpointName.
! !

!TPostcard methodsFor: 'resolve' stamp: 'das 12/2/2005 11:13'!
resolveViewpoint: ogl

	^(ogl harness findViewpointByPostcard: self) ifNil:[ standin ].! !

!TPostcard methodsFor: 'accessing' stamp: 'das 11/14/2005 09:11'!
routerAddress

	^ routerAddress.! !

!TPostcard methodsFor: 'initialize' stamp: 'ar 1/31/2006 18:05'!
routerAddress: address id: id name: name viewpointID: vpid

	routerAddress:= address.
	viewpointName := nil.
	viewpointID := vpid.
	routerName := name.
	routerID := id.
			
! !

!TPostcard methodsFor: 'initialize' stamp: 'ar 1/31/2006 18:05'!
routerAddress: address id: id name: name viewpointName: vpn

	routerAddress:= address.
	viewpointName := vpn.
	viewpointID := nil.
	routerName := name.
	routerID := id.
			
! !

!TPostcard methodsFor: 'accessing' stamp: 'das 11/14/2005 09:11'!
routerID

	^ routerID.! !

!TPostcard methodsFor: 'accessing' stamp: 'das 11/14/2005 09:11'!
routerName

	^ routerName.! !

!TPostcard methodsFor: 'as yet unclassified' stamp: 'das 11/14/2005 12:45'!
standin: stndin

	standin := stndin.! !

!TPostcard methodsFor: 'accessing' stamp: 'das 1/19/2006 16:44'!
viewpointID

	^ viewpointID.! !

!TPostcard methodsFor: 'accessing' stamp: 'das 11/14/2005 09:16'!
viewpointName

	^ viewpointName.! !
TBroadcasterStandalone initialize!
TContactPoint initialize!
TContactPointWS initialize!
